;*****************analog devices aduc7024 init.s********************
;************************15/05/2006*********************************


fiq_stack_size	EQU	128
irq_stack_size	EQU	0
usr_stack_size	EQU	256
swi_stack_size	EQU	0

total_stack		EQU fiq_stack_size + irq_stack_size + usr_stack_size + swi_stack_size

ram_limit		EQU	0x12000				;8kBytes
ram_base		EQU 0x10000

fiq_stack		EQU ram_limit
irq_stack		EQU	fiq_stack - fiq_stack_size
usr_stack	 	EQU	irq_stack - irq_stack_size
swi_stack		EQU	usr_stack - usr_stack_size

usr_mode	EQU 0x10
fiq_mode	EQU 0x11
irq_mode	EQU	0x12

fiq_disable	EQU	0x40
irq_disable EQU	0x80

		REQUIRE8
       	PRESERVE8

		AREA	init, CODE, READONLY, ALIGN=2

        IMPORT  |Image$$RW$$Base|       ; Base of RAM to initialise
        IMPORT  |Image$$ZI$$Base|       ; Base and limit of area
		IMPORT  |Image$$RO$$Limit|
	ARM
	ENTRY

	IMPORT FiqHandler
			
	b	reset
	b	undefined_instruction
software_interrupt
	b	software_interrupt
	b	prefetch_abort
	b	data_abort
	add	r0,r0, #0				;nop
irq_handler
	b	irq_handler
 
	stmfd	r13!, {r0-r7, lr}		; Save some working registers 
	add		r0, pc, #1				; Switch to THUMB state
	BX		r0

	CODE16
	bl		FiqHandler				; Link to the C FiqHandler								

	bx		pc						; Switch back to ARM state
	CODE32
	ldmfd   r13!, {r0-r7,lr}		; Recover temporary saved registers
    subs    pc,lr,#4				; Return from handler
	
	

;===================system debug=================================
data_abort
	mov	r13, #1					;if r13==1 then data_abort axception
	sub r14, r14, #12			;absolute address was make data_abort 
dtab_loop
	b	dtab_loop

prefetch_abort							
	mov r13, #2					;if r13==2 then prefetch_abort axception
	sub r14, r14, #8			;r14 absolute address was make prefetch_abort
prf_loop
	b	prf_loop

undefined_instruction					
	mov r13, #3					;if r13==3 then undefined_instruction axception
	sub r14, r14, #4			;r14 absolute address was make undefined_instruction
un_loop	
	b	un_loop
;===================system debug end=============================

reset

;====================stack initialization========================

	ldr sp, =swi_stack			;swi stackpointer

	mrs r0, cpsr				;enter fiq_mode and set stackpointer
	bic r0, r0, #0x1f
	orr r0, r0, #fiq_mode
	msr CPSR_c, r0
	ldr sp, =fiq_stack
	
	mrs r0, cpsr				;enter irq_mode ant set stackpointer
	bic r0, r0, #0x1f
	orr r0, r0, #irq_mode
	msr CPSR_c, r0
	ldr sp, =irq_stack
	
	mrs r0, cpsr				;finally enter user_mode and set stackpointer
	bic r0, r0, #0x5f		    ;enable fiq
	orr r0, r0, #usr_mode
	msr CPSR_c, r0
	ldr sp, =usr_stack

	ldr r0, =0x00				;stack empty pattern
	ldr r1, =ram_limit
	ldr r2, =ram_limit-total_stack
stack_pattern
	str r0, [r1, #-4]!
	cmp r1, r2
	bne	stack_pattern
;====================stack initialization end====================

;==================Copy data from flash to sram==================
   	ldr r0, =|Image$$RW$$Base|
	ldr r1, =|Image$$ZI$$Base|
	ldr r2, =|Image$$RO$$Limit|
data_copy
	cmp r0,r1
	ldrcc r3, [r2], #4
	strcc r3, [r0], #4
	bne data_copy
 

;====================aduc initialization=========================

;end 
;	b	end

;===================aduc initialization end======================
;	ldr     r0, =into_thumb + 1
;	BX      r0
    
;    THUMB
;	IMPORT test                          ; Next instruction will be Thumb
;into_thumb     
;	BL      test
	IMPORT C_Entry
;	ldr r0, test + 1 
	b  C_Entry
	END