/*########################################################
####										-ilioslab-						  			####
####	              		 25/02/10   				    			####
########################################################*/


#include "aduc7024.h"
#include "epson6610.h"
#include "math.h"

//main functions
void Measurements(void);
void Initialization(void);
void SetFrequency(void);
void SetMode(void);
void SetAD9833(unsigned long value , unsigned int mode);
void BlinkDigit(unsigned char on_off);
void screen1(void);
void InitSW(void);
void InitADC(void);
void FindVoltage(void);
void ShowVoltage(int voltage);
void ShowDutyCycle(void);
void SetButton(void);
void ModeButton(void);
void SweepButton(void);
void BlinkSweep(unsigned char param,unsigned char digit,unsigned char onoff);
void ParamCalculations(void);
void FiqHandler(void);
void CalibrationButton(void);
void CalibrationSet(char type);
void PowerCycles(void);

//definitions
#define  multiplier 	(*(volatile unsigned int *) 0X8f200)			//value in flash be used for frequency calculations. After factory calibration filled with fmultiplier
#define  fmultiplier		234562																	//factory precalculated multiplier = ((2^28 / ad9833clock)<<16) for 75MHz oscillator=234562. 
//lcd contrast
#define  LCDCONTRAST 	(*(volatile unsigned short *) 0X8f400)		//value in flash used for lcd contrast

//AD9834 chip definitions 
#define	 RESET				0x00000100
#define	 SINUSF0P0		0x00002000
#define	 SINUSF0P1		0x00002400
#define	 SINUSF1P0		0x00002800
#define	 SINUSF1P1		0x00002c00
#define	 TRIANGLEF0P0	0x00002002
#define	 TRIANGLEF0P1	0x00002402
#define	 TRIANGLEF1P0	0x00002802
#define	 TRIANGLEF1P1	0x00002c02
#define	 SQUAREF0P0		0x00002028
#define	 SQUAREF0P1		0x00002428
#define	 SQUAREF1P0		0x00002828
#define	 SQUAREF1P1		0x00002c28
#define	 CONTROL			0x00000000
#define	 PHASE0				0x0000c000
#define	 PHASE1				0x0000e000
#define	 FREQ0				0x00004000
#define  FREQ1				0x00008000

//A/D Measurements definitions
#define   GROUND 		(*(volatile unsigned short *) 	0X8f600)		  //flash memory placed 0,00V value being used by FintVoltage() 
#define   TWELVEVOLT 	(*(volatile unsigned short *) 	0X8f602)		//flash memory placed 12,00V value being used by FintVoltage() 
#define   STEP		 	(*(volatile unsigned short *) 	0X8f604)			//flace memory placed step value 
#define   FGROUND 			0x08b7																		//factory calibrated 0,00V(ground) value
#define   FTWELVEVOLT 	0x0130																		//factory calibrated 12,00v value
#define   FSTEP		 			0x9f6b				   													//(1200 << 16)/(GROUND - TWELVEVOLT)								
#define   NMBROFSMBLS	25600/(CD+1)		  													//the number of samples per measurement is proportionable to core clock 

//Blinking definitions
#define	  ON			0x00																						//show digit 																						
#define	  OFF			0x01																						//hide digit
#define	  ONOFFTIME		200000/(CD+1)		  													//stable on/off time regardless of the core clock

//Power cycles
#define   POWERON 		(*(volatile unsigned short *) 	0X8f000);		//the point in flash where are stored power cycles 

//Variables:
//main frequency
unsigned char digit[7];																						//output frequency value. digit[0]=monads,digit[1]=tens,digit[2]=hundreds,.... 
unsigned char mode=0;																							//output waveform type. 0->sinus 1->triangle 2->square
unsigned char pnt;																								//pointer to digit being changed in main frequency array
//measurements
unsigned short maximum=0, minimum=0,duty=0; 											//used for measurements
int offset=0;
unsigned int dutycycle=0,counter=0;																//counting for measuremets
//sweep mode
int sweep_onoff=0;
unsigned int sweeparam[3][7];																			//start,stop,time values
unsigned long long currentfrequency; 															//for c compiler umull
unsigned int ticks,start,stop,step,time,plustep=0,doubling=0;			//from array to variable
unsigned char shift;																							//variable shifting for better accurancy
unsigned char  freg0freg1=0;
unsigned char  linearlog=0;
static const unsigned char log2[101] =								 						//100 different values per octave
	 {0x80, 0x80, 0x81, 0x82, 0x83, 0x84, 0x85, 0x86, 0x87, 0x88, 										   
	  0x89, 0x8a, 0x8b, 0x8c, 0x8d, 0x8e, 0x8f, 0x90, 0x91, 0x92,
	  0x93, 0x94, 0x95, 0x96, 0x97, 0x98, 0x99, 0x9a, 0x9b, 0x9c,
	  0x9d, 0x9e, 0x9f, 0xa1, 0xa2, 0xa3, 0xa4, 0xa5, 0xa6, 0xa7,
	  0xa8, 0xaa, 0xab, 0xac, 0xad, 0xae, 0xb0, 0xb1, 0xb2, 0xb3,
	  0xb5, 0xb6, 0xb7, 0xb8, 0xba, 0xbb, 0xbc, 0xbe, 0xbf, 0xc0,
	  0xc2, 0xc3, 0xc4, 0xc6, 0xc7, 0xc8, 0xca, 0xcb, 0xcc, 0xce,
	  0xcf, 0xd1, 0xd2, 0xd4, 0xd5, 0xd7, 0xd8, 0xda, 0xdb, 0xdd,
	  0xdf, 0xe0, 0xe1, 0xe3, 0xe5, 0xe6, 0xe8, 0xe9, 0xeb, 0xed,
	  0xef, 0xf0, 0xf2, 0xf3, 0xf5, 0xf7, 0xf8, 0xfa, 0xfc, 0xfe,0xff};

int contrast=0x24;
	  						
//Screen strings
byte *clearline="                ";
byte *squarelastline="DutyC";
byte *initscreen="Hz  Measurements             Max             Pk-Pk           Min";
byte *mode0="sine ";
byte *mode1="square";
byte *mode2="triangle";
byte *clipping="clipping";
byte *shiftscreen="   sweep stop   start:          stop:   	       msec:           mode:";
byte *linear="linear";
byte *logarithic="log   ";
byte *run="run ";
byte *stp="stop";
//calibration strings
byte *calibration=" calibration                    >LcdContrast     Measurements    Frequency       FactoryPreset";
byte *stringcontrast="press + - button     press        calibration       to  exit";
byte *measure1="set low        voltage at        00.00v      and then press    set button";
byte *measure2="    set high       voltage at        12.00v      and then press    set button";
byte *measure3="  calibration       complete         press        calibration       for exit";
byte *factory1=" press set for    calibration    or calibration     for exit";
byte *factory2="  calibration       complete         press        calibration       for exit";
byte *frequency1="                  with up and     down button    set the output   frequency at     100.000 Hz    then press set";
byte *frequency2="  calibration       complete         press        calibration       for exit  ";
//boot screen
byte *firmware="firmware: 1.0";
byte *powercycles="pwrc:";
byte *alingment="123";

