
/*########################################################
####									-ilioslab-						  				####
####	             		  29/01/08   				      			####
########################################################*/
 
#include "main.h"

/*--------------------------------------------------------------------------------------------
  Name         : C_Entry
  Description  : C entry point function 
  Time		   	 :  
  Notes        : push buttons polling method is used 
---------------------------------------------------------------------------------------------*/
void C_Entry(void)
{						
	Initialization();
	
// polling push buttons
	while(1){																					//watching for push
		switch (GP4DAT & 0x000000ff) {
			case 0xff :																		//nothing was pushed,
				Measurements();															//calculate output voltage and print them on screen
				break;
			case 0xfe :																		//set pushed
				SetButton();
				break;
			case  0xdf :  																//mode(wave) pushed 
				ModeButton();																				
		  		break;				 							
			case 0xf7 :																		// +	pushed
				if((mode == 1) & (DAC0DAT > 0x00800000))		//if square wave ++square duty cycle 
				DAC0DAT -= 0x100000;
				Delay(200000);
				break;
			case 0xef :																		// - pushed
				if((mode == 1) & (DAC0DAT < 0x0bff0000))		//if square wave --square duty cycle
					DAC0DAT += 0x100000;
				Delay(200000);
				break;
			case 0x7f :																		//calibration  pushed
				CalibrationButton();											
				break;
			case 0xbf :																		//sweep pushed
				SweepButton();
				Delay(300000);
				break;
			default :
				break;
		}	
	}
}

/*--------------------------------------------------------------------------------------------
  Name         :  ModeButton
  Description  :  Change the output wave, sinus->square->triangle
  Time		   	 :  11/06/2009
  Notes        :  The frequency value remain unchangable
---------------------------------------------------------------------------------------------*/
void ModeButton(void)
{
	if(mode < 2)									  	
		++mode; 
	else
		mode = 0;
	SetMode();
	Delay(500000);
}

/*--------------------------------------------------------------------------------------------
  Name         :  SetMode
  Description  :  change waveform (sinus,triangle,square) and update lcd
  Time		   	 :  03/06/2009
  Notes        :  The frequency value remain unchangable
---------------------------------------------------------------------------------------------*/
void SetMode(void)
{
	 unsigned int i;

	 if(mode == 0){																		//sinus wave						
	 	 SetAD9833( SINUSF0P0 , CONTROL);
		 GP1SET = 0x00040000;														//select ADG779 signal path
		 DAC0CON = 0x02;																//clear DAC, reduse noise. Only one signal path on ADG779
		 }
	 else{ 
	 	if(mode == 1){																	//square wave 
		  	SetAD9833( TRIANGLEF0P0 , CONTROL);					//used external circuit comparator for "square from triangle"
			GP1CLR = 0x00040000;									 				//select ADG779 signal path
			DAC0CON = 0x12;
			DAC0DAT = 0x06000000;													//set Duty Cycle 50%
		 }
		else {																					//triangle wave
		 	if(mode == 2){
					SetAD9833( TRIANGLEF0P0 , CONTROL);				//triangle output from AD9834
					GP1SET = 0x00040000;											//select ADG779 sinus signal path 
					DAC0CON = 0x02;														//clear DAC, reduse noise. Only one signal path on ADG779
			}
		}
	}

	LcdGotoXY(1,1);											   						//clear first lcd's line
	for(i=0; i<16; i++)
    	LcdChr(FONT_1X,' ');
	LcdUpdate();

	switch (mode){																		
		case 0 :																				//sinus wave
			LcdGotoXY(6,1);																
			LcdStr(FONT_1X,mode0);
			LcdGotoXY(1,8);
			LcdStr(FONT_1X,clearline);
			break;			
		case 1 :																				//square wave
			LcdGotoXY(5,1);
			LcdStr(FONT_1X,mode1);
			if(sweep_onoff == 0){
				LcdGotoXY(12,8);
				LcdStr(FONT_1X,squarelastline);							//dutycycle information 
			}
			break;
		case 2 :																				//triangle wave
			LcdGotoXY(5,1);
			LcdStr(FONT_1X,mode2);
			LcdGotoXY(1,8);
			LcdStr(FONT_1X,clearline);
			break;	
	}
	LcdUpdate();
}

/*--------------------------------------------------------------------------------------------
  Name         :  SetButton
  Description  :  Set button action. 
  Time		   	 :  06/2009
  Notes        :  Works only if set button pushed at first screen.
---------------------------------------------------------------------------------------------*/
void SetButton(void)
{
	unsigned int on_off=0;

  BlinkDigit(OFF);												//hide digit on entry 
	Delay(300000);
	while(GP4DAT != 0xfe){									//loop until set button be pushed again 	
		switch (GP4DAT & 0x000000ff) {
			case 0xff :													//nothing pushed
				++on_off;													//increace countrer
				if(on_off == ONOFFTIME)						
					BlinkDigit(ON);									//show digit
				if(on_off > (ONOFFTIME * 2)){
					BlinkDigit(OFF);								//hide digit 
					on_off = 0;											//reset counter
				}											
				break;
			case 0xfd :													//right button pushed
				BlinkDigit(ON);										//print on screen previous digit
				if(pnt > 0)												//select next digit(cursor moved right)
					--pnt;
				else
					pnt = 6;										
				BlinkDigit(OFF);									//hide digit
				on_off = 0;												//reset counter
				Delay(300000);
				break;
			case 0xfb :													//left button poushed
				BlinkDigit(ON);										//print on screen previous digit										
				if(pnt < 6)												//select next digit(cursor moved left)
					++pnt;
				else
					pnt = 0;
				BlinkDigit(OFF);									//hide digit
				on_off = 0;												//reset counter
				Delay(300000);
				break;
			case 0xf7 :													//up(+) button pushed 
				if(digit[pnt] < 9)			   				//change frequency 1..9. Do not set frequency until set button pushed.
					++digit[pnt]; 
				else
					digit[pnt] = 0;
				BlinkDigit(ON);
				on_off = 0;
				Delay(150000);
				break;
			case 0xef :													//down(-) button pushed
				if(digit[pnt] > 0)			   				//change frequency 1..9. Do not set frequency until set button pushed.
					--digit[pnt]; 
				else
					digit[pnt] = 9;
				BlinkDigit(ON);
				on_off = 0;
				Delay(150000);
				break;
			case  0xdf :  											//mode button pushed 
				ModeButton();											//change and set output waveform																				
		  		break;
			default :
				break;

		}
	}
	BlinkDigit(ON);												//show digit on exit
	SetFrequency();												//set frequency on exit(set button pushed again)
	Delay(300000);
}

/*--------------------------------------------------------------------------------------------
  Name         :  BlinkDigit
  Description  :  Frequency digit being changed blinking
  Time		   	 :  06/2009
  Notes        :  It works only on frequency digits
---------------------------------------------------------------------------------------------*/	
void BlinkDigit(unsigned char on_off)
{
	LcdGotoXY((7 * 2) - 1 - (2 * pnt) ,3);  //digit being changed
	if(on_off == 0 )
		LcdChr(FONT_2X,digit[pnt]+32+16);			//show digit
	else
    	LcdChr(FONT_2X,' ');								//hide digit
	LcdUpdate();
}

/*--------------------------------------------------------------------------------------------
  Name         :  Calibration 
  Description  :  main calibration screen menu entering 
  Time		     :  03/01/2010
  Notes        :  +,- button used to change the type of calibration. Calibration button for exit,
									set button(CalibrationSet() function entry) to start the calibration procedure.
---------------------------------------------------------------------------------------------*/	
void CalibrationButton(void)
{
	int type = 0;																	//0=LcdContrast, 1=measuremets, 2=frequency, 3=factory preset
	
	Delay(200000);																//button glitch delay
	LcdClear();
	LcdGotoXY(1,1);
	LcdStr(FONT_1X,calibration);
	LcdUpdate();

	SetAD9833( SINUSF0P0 , CONTROL);							//set sinus
	GP1SET = 0x00040000;													//select ADG779 signal path
	DAC0CON = 0x02;																//clear dac, reduce noise
	SetAD9833((1000 * multiplier)>>16 , FREQ0);		//set output frequency 1KHz

	while(GP4DAT != 0x7f){												//loop while calibration button pushed
		switch (GP4DAT & 0x000000ff){
			case 0xff :																//nothing pushed											
				break;																	//do nothing
			case 0xfe :																//set button pushed
				CalibrationSet(type);
				break;			 							
			case 0xf7 :																//+ button pushed
				LcdGotoXY(1,type+3);
				LcdChr(FONT_1X,' ');
				if(type > 0)														//change type of calibration
					type--;																//move ">" character up 
				else
					type = 3;
				LcdGotoXY(1,type+3);
				LcdChr(FONT_1X,'>');										//print ">" character on sceen
				LcdUpdate();
				Delay(200000);													//without this, fast +
				break;
			case 0xef :																//- button pushed
				LcdGotoXY(1,type+3);
				LcdChr(FONT_1X,' ');
				if(type < 3)														//change type of calibration
					type++;																//move ">" character down
				else
					type = 0;
				LcdGotoXY(1,type+3);
				LcdChr(FONT_1X,'>');										//print ">" character on sceen
				LcdUpdate();
				Delay(200000);													//without this, fast -
				break;
			default :
				break;
		}	
	}
	LcdClear();
	SetFrequency();
	SetMode();
	ADCCON = 0x4a4;
	Delay(10);
	LcdGotoXY(15,3);
	LcdStr(FONT_1X,initscreen);
	LcdUpdate();
	Delay(200000);										
}

/*--------------------------------------------------------------------------------------------
  Name         :  CalibrationSet
  Description  :  
  Time		   	 :  03/01/2010
  Notes        :  
---------------------------------------------------------------------------------------------*/
void CalibrationSet(char type)
{
	unsigned int i,temp=0;					 				//temp for the measuremets
	int temp1=0;														//temp1 for multiplier increment-decrement
	unsigned long long temp2 = 0;						//temp2 temporary fmultiplier value

	minimum = 0xffff;
	maximum = 0x0000;
	FEEMOD = 0x08;									   			//can read,write the flash memory
	
//LcdContrast calibration
	if(type == 0){												 
		LcdClear();
		LcdGotoXY(1,2);
		LcdStr(FONT_1X,stringcontrast);				//"press + - button press calibration to exit";
		LcdGotoXY(1,7);
		LcdStr(FONT_1X,clearline);
		LcdGotoXY(1,7);
		temp1 = LCDCONTRAST & 0x7f;						// "#####    " screen effect
		while(temp1 > 0){
			LcdChr(FONT_1X,'#');
			temp1 = temp1 - 8;
		}
		LcdUpdate();
		contrast = LCDCONTRAST & 0x7f;				
		while(GP4DAT != 0x7f){								//while not calibration button pushed
			switch (GP4DAT & 0x000000ff){
				case 0xff :												//nothing poush											
					break;													//do nothing
				case 0xf7 :												//+ button pushed 
					if(contrast < 127)
						contrast++;										//increase contrast 
					WriteLcdCommand(SETCON);
					WriteLcdData(contrast); 				//and write new value to lcd register
					temp1 = contrast;
					LcdGotoXY(1,7);
					LcdStr(FONT_1X,clearline);
					LcdGotoXY(1,7);
					while(temp1 > 0){					    	//increase contrast bar "######..."
						LcdChr(FONT_1X,'#');
						temp1 = temp1 - 8;
					}
					LcdUpdate();
					Delay(100000);									//delay for continuous poushing
					break;
				case 0xef :												//- button pushed
					if(contrast > 0)
						contrast--;										//decrease contrast
					WriteLcdCommand(SETCON);
					WriteLcdData(contrast); 				//and write new value to lcd register
					temp1 = contrast;
					LcdGotoXY(1,7);
					LcdStr(FONT_1X,clearline);
					LcdGotoXY(1,7);							
					while(temp1 > 0){								//decrease contrast bar "####....."
						LcdChr(FONT_1X,'#');
						temp1 = temp1 - 8;
					}
					LcdUpdate();
					Delay(100000);
					break;
				default :
					break;
			}
		}
		FEEDAT = contrast;								   	//erase page and write contrast value
		FEEADR = 0x8f400;											//Write new contrast value to ADUC7024 flash memory
		FEECON = 0x03;												//erase page and write
		Delay(50000);
	}

//measurement calibration
	if(type == 1){										  
		LcdClear();
	 	LcdGotoXY(5,2);
		LcdStr(FONT_1X,measure1);								//"set low voltage at 00.00v and then press set button"
		LcdUpdate();
		Delay(400000);									   			//for next instruction
		while(GP4DAT != 0xfe);									//wait until set button pushed
		for(i=0;i<20000;i++){							   		//take 20000 measurements
			while(ADCSTA == 0x00);
			temp = (ADCDAT >> 16);
			if(temp > maximum)							   		//calculate maximum(00.00v value)
			maximum = temp;
		}
		FEEDAT = maximum;								   			//write 00.00v value to flash
		FEEADR = 0x8f600;
		FEECON = 0x03;													//erase page and write
		Delay(200000);									  			//for flash delay

		LcdClear();
		LcdGotoXY(1,2); 
		LcdStr(FONT_1X,measure2);								//"set high voltage at 12.00v and then press set button"
		LcdUpdate();	
		while(GP4DAT != 0xfe);							  	//wait until set button pushed
		for(i=0;i<20000;i++){										//take 20000 measurements
			while(ADCSTA == 0x00);
			temp = (ADCDAT >> 16);
			if(temp < minimum)							 			//calculate minimum(12.00v value)
			minimum = temp;
		}
		FEEDAT = minimum;												//write 12.00v value to flash
		FEEADR = 0x8f602;
		FEECON = 0x02;
		Delay(100);
		
		FEEDAT = 78643200/(maximum - minimum);	//calculate STEP to 10mV base
		FEEADR = 0x8f604;												//write STEP value to flash
		FEECON = 0x02;
		
		LcdClear();															//clear lcd
		LcdGotoXY(1,2);
		LcdStr(FONT_1X,measure3);								//"calibration complete press calibration for exit"							
		LcdUpdate();
		while(GP4DAT != 0x7f);							 		//wait until calibration button pushed to exit 
	}

//multiplier(output frequency accurancy) calibration
	if(type == 2){											
		temp2 = fmultiplier; 
		SetAD9833((100000 * temp2)>>16 , FREQ0);														//set output sinus to 100KHz
		LcdClear();
		LcdGotoXY(1,1);
		LcdStr(FONT_1X,frequency1);																					//"with up and down button set the output frequency at 100.000 Hz then press set";
		LcdUpdate();								 
		Delay(300000); 																											//for next instruction
		while(GP4DAT != 0xfe){																							//while not set button pushed
			while((GP4DAT != 0xfe) && (GP4DAT != 0xf7) && (GP4DAT != 0xef));	//wait until +,-,set buttons pushed
			if(GP4DAT == 0xf7){																								//+ button pushed
				++temp1;
	 			SetAD9833((100000 * (temp2 + temp1))>>16 , FREQ0);		  				//increase output frequency
				Delay(300000);											  													//for fast ++
			}
			if(GP4DAT == 0xef){																								//- button pushed
				--temp1;
	 			SetAD9833((100000 * (temp2 + temp1))>>16 , FREQ0);							//degrease output frequency
				Delay(300000);																									//fast -- 
			}
		}													
		temp1 = (fmultiplier + temp1);	 		  															//here after set button
		FEEDAT = temp1 & 0x0000ffff;																				//write new multiplier in flash
		FEEADR = 0x8f200;
		FEECON = 0x03;
		Delay(300000);
		FEEDAT = ((temp1 & 0xffff0000)>>16);
		FEEADR = 0x8f202;
		FEECON = 0x02;
		Delay(100);

		LcdClear();
		LcdGotoXY(1,2);
		LcdStr(FONT_1X,frequency2);																				//"calibration complete press calibration for exit"
		LcdUpdate();
		while(GP4DAT != 0x7f);						  															//wait until calibration button
	}

//factory preset calibration
	if(type == 3){											
		LcdClear();
		LcdGotoXY(1,2);
		LcdStr(FONT_1X,factory1);												//"press set for calibration or calibration for exit"
		LcdUpdate();
		Delay(300000);									   							//next instruction
		while((GP4DAT != 0xfe) && (GP4DAT != 0x7f));		//wait until set or calibration button
		if(GP4DAT == 0xfe){															//if set button

			FEEDAT = FGROUND;															//factory measuremet values from flash to flash
			FEEADR = 0x8f600;
			FEECON = 0x03;
			Delay(300000); 		
			FEEDAT = FTWELVEVOLT;
			FEEADR = 0x8f602;
			FEECON = 0x02;
			Delay(100);
			FEEDAT = FSTEP;
			FEEADR = 0x8f604;
			FEECON = 0x02;
			Delay(100);
			FEEDAT = fmultiplier & 0x0000ffff;						//factory multiplier value
			FEEADR = 0x8f200;
			FEECON = 0x03;
			Delay(300000);
			FEEDAT = ((fmultiplier & 0xffff0000)>>16);	  //attention 32bit to 16bit
			FEEADR = 0x8f202;
			FEECON = 0x02;
			Delay(100);
			FEEDAT = 0x38;								   							//factory lcd contrast value
			FEEADR = 0x8f400;								
			FEECON = 0x03;									
			Delay(300000);
			WriteLcdCommand(SETCON);
			WriteLcdData(LCDCONTRAST); 	
//			WriteLcdData(3);

			LcdClear();
			LcdGotoXY(1,2);
			LcdStr(FONT_1X,factory2);
			LcdUpdate();
			while(GP4DAT != 0x7f);						  					//wait until calibration button
		}
	}	
		
	FEEMOD = 0x00;										    						//lock flash memory

}

/*--------------------------------------------------------------------------------------------
  Name         :  SweepButton
  Description  :  Run when sweepbutton pushed. New polling for buttons: set,+,-,right,left,mode.
									If sweep run then fiq interrupt from timer2 every 0,122msec. Global sweeparam[3][7]
									unsint for parameters start,stop,time(msec).
  Time		   	 :  02/08/2009
  Notes        :  
---------------------------------------------------------------------------------------------*/	
void SweepButton(void)
{
	unsigned int param,digit;
	unsigned int on_off=0;
	
	sweep_onoff = 1;
	LcdClear();
	LcdGotoXY(1,2);
	LcdStr(FONT_1X,shiftscreen);												//"sweep stop   start:   stop:  msec:   mode:"
	LcdUpdate();

	for(param=0; param<3; param++){
		LcdGotoXY(8,(3+param));														//print sweep values to lcd 
	  	for(digit=0; digit<7; digit++)
	  	  LcdChr(FONT_1X,sweeparam[param][6-digit]+32+16);
	}
	
	LcdGotoXY(9,6);								   
	if(linearlog & 0x01)								 								//print "linear" on lcd
		LcdStr(FONT_1X,linear);
	else
		LcdStr(FONT_1X,logarithic);												//print "log   " on lcd

	param = 0;
	digit = 0;
	LcdUpdate();	
		
	Delay(300000);

	while(GP4DAT != 0xbf){															//while not push shift button	
		switch (GP4DAT & 0x000000ff) {
			case 0xff :																			//nothing pushed
				if(param != 4){																//then blinking if sweep isn't at running state
					++on_off;								
					if(on_off == ONOFFTIME)											//if CD=1 22Mhz ONOFFTIME/2
						BlinkSweep(param,digit,ON);			
					if(on_off > (ONOFFTIME * 2)){
						BlinkSweep(param,digit,OFF);
						on_off = 0;
					}		
				}						
				break;
			case 0xfe :																			//set button pushed
				BlinkSweep(param,digit,ON);
				if(param == 3){																//if true run sweep
					ParamCalculations();
					LcdGotoXY(10,2);
					LcdStr(FONT_1X,run);
					LcdUpdate();					
					T2LD = 0x03;																//timer2 value = 3
					T2CON = 0x4c0;															//internal oscillator,count down,enable timer,periodic mode,binary,oscillator/1
					FIQEN = 0x10;						    								//enable interrupt from timer2
				}																							//timer count 3,2,1,0interrupt,3,2,1,0interrupt...//interrupts interval:4clock=1/8192sec=0,122msec
				if(param < 4)								
					++param;
				else{																					//new parameters value
					param = 0;
					T2CON = 0x440;															//disable timer2
					FIQCLR = 0x10;															//clear time2 interrupt
					LcdGotoXY(10,2);
					LcdStr(FONT_1X,stp);
					LcdUpdate();
				}
				Delay(300000);
				break;
			case 0xf7 :																			//poush +
				if(param < 3){
					if(sweeparam[param][digit] < 9)			
						++sweeparam[param][digit];
					else
						sweeparam[param][digit]=0;
					BlinkSweep(param,digit,ON);
					on_off = 0;
				}
				if(param == 3){								 							//linear log
					linearlog++;
					BlinkSweep(param,digit,ON);
					on_off = 0;
				}
				Delay(150000);
				break;
			case 0xef :																		//poush -
				 if(param < 3){
				 	if(sweeparam[param][digit] > 0)
						--sweeparam[param][digit];
					else
						sweeparam[param][digit]=9;
					BlinkSweep(param,digit,ON);
					on_off = 0;
				}										  
				if(param == 3){							  							//linear log
					--linearlog;
					BlinkSweep(param,digit,ON);
					on_off = 0;
				}
				Delay(150000);
				break;
			case 0xfd :																		//poush right
				BlinkSweep(param,digit,ON);
				if(digit > 0)
					--digit;
				else
					digit = 6;
				BlinkSweep(param,digit,OFF);								//current digit
				on_off = 0;
				Delay(300000);
				break;
			case 0xfb :																		//poush left
				BlinkSweep(param,digit,ON);
				if(digit < 6)
					++digit;
				else
					digit = 0;
				BlinkSweep(param,digit,OFF);								//current digit
				on_off = 0;
				Delay(300000);
				break;
			case  0xdf :  																//poush mode 
				ModeButton();																				
		  		break;
			default :																		
		  		break;				
		}
	}
	
	sweep_onoff = 0;																	//exit from sweep
	T2CON = 0x440;																		//disable timer2
	FIQCLR = 0x10;																		//disable interrupt
	LcdClear();
	SetFrequency();
	SetMode();
	LcdGotoXY(15,3);
	LcdStr(FONT_1X,initscreen);
	LcdUpdate();
	Delay(300000);
}

/*--------------------------------------------------------------------------------------------
  Name         :  BlinkSweep
  Description  :  Blink active digit on start[] or stop[] or time[]. 
  Time		   	 :  02/08/2009
  Notes        :  Works in sweep mode only.
---------------------------------------------------------------------------------------------*/
void BlinkSweep(unsigned char param,unsigned char digit,unsigned char onoff)
{
	unsigned int i;

	if(param < 3){										  	//start[],stop[],time[] values blinking
		LcdGotoXY(14-digit,param+3);
		if(onoff == 0)
			LcdChr(FONT_1X,sweeparam[param][digit]+32+16);		//print active digit on screen	
		else
    		LcdChr(FONT_1X,' ');														//print " " on screen						
	}
	if(param == 3){		  									//linear,log blinking
		LcdGotoXY(9,6);
		if(onoff == 0){
			if(linearlog & 0x01)
				LcdStr(FONT_1X,linear);					//print "linear" on screen					
			else
				LcdStr(FONT_1X,logarithic);			//print "log   " on screen
		}else{
			for(i=0;i<7;i++)
				LcdChr(FONT_1X,' ');						//print " " on screen
		}
	}
	LcdUpdate();	
}

/*--------------------------------------------------------------------------------------------
  Name         :  ParamCalculations
  Description  :  From array sweeparam[3][7] to variables start,stop,time. Step,shift production.
  Time		     :  02/08/2009
  Notes        :  32bit variables and the algorith is more accurate than internal
									oscillator.No need for 64bit variables. 
---------------------------------------------------------------------------------------------*/
void ParamCalculations(void)
{
	unsigned int i;
	unsigned int temp1,temp2,octave=0;
	int m = 1;

	start = 0;
	for(i=0; i<7; i++){										//calculate start frequency
	  	start += (sweeparam[0][i] * m);
		m = m * 10;
	 }
	
	m = 1;
	stop = 0; 
	for(i=0; i<7; i++){										//calculate stop frequency
	  	stop += (sweeparam[1][i] * m);
		m = m * 10;
	 }
	
	m = 1;
	time = 0;
	for(i=0; i<7; i++){										//calculate total time of sweep in miliseconds
	  	time += (sweeparam[2][i] * m);
		m = m * 10;
	 }
	 
//!!!Viariable shifting. Step as long as possible. e.g start=100
//stop=200 step=((200-100)<<bigshift)/ticks for better accuracy
//!!!32768/4=8,192tick/msec 8,192*32=0x106
//Stable 5bit shifting.Small but ok accuracy without overflow.
//step 32bit variable for smaller code size(-0.7Kbytes) than 64bit
	 shift = 0;														
	if(stop >= start){										
		ticks = (time*0x106)>>5;						
	 	step = stop - start;								
		while(step<0x80000000){							
	 		step <<= 1;
			shift++;													//shift for plustep variable
		}
		step = step/ticks;									//math.h included for this line
		currentfrequency = start;
		plustep = start;
	}								
	else{																	//for stop -> start sweep.
		ticks = (time*0x106)>>5;						//10000000*8,192*64=0x138800000 overflow.
		step = start - stop;
		while(step<0x8000000){
	 		step <<=1;
			shift++;
		}
		step = step/ticks;
		currentfrequency = stop;
		plustep = stop;	
	}
	if(!(linearlog & 0x01)){								//logarithmic sweep
		temp1 = start;
		if(start > stop){						   				//start=stop, stop=start
			start = stop;
			stop = temp1;
		}			
		while(temp1	< stop){
			temp1 <<= 1;
			octave++;
		}
		--octave;
		temp1 >>=1;
		octave *=100;

		i=0;
		temp2 = temp1;
		while(temp2 < stop){
			temp2 = ((temp1 * log2[i])>>7);
			i++;
		}			
		octave += (i-2);
		doubling = (ticks*100)/octave;
		currentfrequency = start;
		ticks = 0;
		plustep = start;
	}
}

/*--------------------------------------------------------------------------------------------
  Name         :  FiqHandler
  Description  :  Run every one tick=0.122msec. Works only in sweep. Use this method(interrupt)
									for stable timing from timer2(internal oscillator 32768Hz)   
  Time		   	 :  02/08/2009
  Notes        :  
---------------------------------------------------------------------------------------------*/
void FiqHandler(void)
{
	unsigned long long freqregister = 0;

//	GP1SET = 0x00020000;															//oscilloscope debugging for FiqHadler time interval 
	freqregister = (currentfrequency * multiplier);			//calculate AD9833 28bit frequency register
	if((freg0freg1 & 0x01) == 0){												//this is only way to make a sweep frequency. See AD9834 manual
		SetAD9833(freqregister >> 16 , FREQ0);
		if(mode == 2) 
			SetAD9833( TRIANGLEF0P0 , CONTROL);
		else
			SetAD9833( SINUSF0P0 , CONTROL);
	} 
	else {
		SetAD9833(freqregister >> 16 , FREQ1);
		if(mode == 2)
			SetAD9833( TRIANGLEF0P0 , CONTROL);
		else
			SetAD9833( SINUSF1P0 , CONTROL);
	}	
	++freg0freg1;																			//if fr even number load FREQ0 else load FREQ1
	
	if(linearlog & 0x01){
		if(stop >= start){		
			if(currentfrequency < stop){
				plustep +=step;
				currentfrequency = start+(plustep>>shift);		//see function ParamCalculation
			}
			else{
				currentfrequency = start;
				plustep = 0;
			}
		}else{
			if(currentfrequency > stop){
				plustep +=step;
				currentfrequency = start-(plustep>>shift);	   
			}
			else{
				currentfrequency = start;
				plustep = 0;
			}
	 	}
	}else{
		if(currentfrequency < stop){
			if(ticks < doubling){
				currentfrequency = (log2[(ticks*100)/doubling]*plustep)>>7;
				ticks++;
			}else{
				plustep <<=1;
				ticks = 0;
			}
		}else{
			currentfrequency = start;
			plustep = start;
			ticks = 0;
		}
	} 

//   GP1CLR = 0x00020000;							//for debuging
   T2CLRI = 0xff;											//clear interrupt
}

/*--------------------------------------------------------------------------------------------
  Name         :  Measurements
  Description  :  take 25600 samples, fill maximum with the bigest sample, minimum with the smallest sample
  Time		   	 :  11/06/2009
  Notes        :  show measurements after NMBROFSMBLS=25600 cycles
---------------------------------------------------------------------------------------------*/	
void Measurements(void)
{
	unsigned int temp;

	counter++;

	while(ADCSTA == 0x00);
	temp = (ADCDAT >> 16);

	if(temp > maximum)														//find maximum sample
		maximum = temp;
						 
	if(temp < minimum)														//find minimum sample
		minimum = temp;

//After any measurement(total 25600(41.78Mhz clock) for easy calculation), increase dutycycle 
//value if dutycycle>duty(output square wave midpoint). After that shift dutycycle>>8.
//This give 0<%dutycycle<100.	
	if(temp <= duty)															//find DutyCycle for square wave
		++dutycycle;															 
																								
									
																		
	if(counter >= NMBROFSMBLS){										//(25600/8)=100	for easy dutycycle calculation
		duty = maximum - ((maximum - minimum)>>1); 	//find midpoint of the square wave
		FindVoltage();															//transform ADC sample to real voltage 
		counter = 0;																//prepare for next measurement cycle
		if(mode == 1){
			ShowDutyCycle();													//transform ADC samples to %dutycycle
		}
		dutycycle = 0;															//prepare for next measurement cycle
	}
}

/*--------------------------------------------------------------------------------------------
  Name         :  FintVoltage
  Description  :  transform ADC register sample values to voltage values. 
  Time		   	 :  06/2009
  Notes        :  examble: After level shifting circuit have been measured by ADC FGROUND(0Volts)=0x8b7,
									FTWELVEVOLT(12Volts)=0x130. Then ADC LSB=12volt/(0x8b7-0x130)=0,006227Volts=6,227mVolts=
									=0,6227x10mVolts. 10mVolts measuremets base being used. But the number 0,6227 is decimal.
									Need not floating point arithmetic and division if the base have been shifted <<16(base*2^16).
									Then FSTEP=0,62272963<<16=40811=0x9f6b. Then every value(like low,high) that multiplied with 
									FSTEP must sfifted >>16.
---------------------------------------------------------------------------------------------*/							
void  FindVoltage(void)
{
	int low=0,high=0;

	low = (((GROUND - maximum) * STEP) >> 16) & 0xfffffffe;		//low calculated from maximum(reciprocal from level shifting circuit) 			
	maximum = 0;																							//reset maximum for next measurent cycle
	high = (((GROUND - minimum) * STEP) >> 16) & 0xfffffffe;  //high calculated from minimum(reciprocal from level shifting circuit)
	minimum = 0xffff;																					//reset minimum for next measurent cycle

	offset = (low - ((low - high)>>1))+16;                    //midpoint calculation

	LcdGotoXY(1,5);																					  	
	if(high < 1300){		
		ShowVoltage(high);																			//print high on screen										
		LcdChr(FONT_1X,' ');
	}
	else
		LcdStr(FONT_1X,clipping);																//if high>13Volts print clipping on high
	LcdGotoXY(1,6);
	ShowVoltage(high - low);																	//print pK-pK on screen
	LcdGotoXY(1,7);
	ShowVoltage(low);																					//print low on screen
	LcdUpdate();
}

/*--------------------------------------------------------------------------------------------
  Name         :  ShowVoltage
  Description  :  display high, Pk-PK and low value to lcd
  Time		   	 :  06/2009
  Notes        :  
---------------------------------------------------------------------------------------------*/	 
void ShowVoltage(int voltage)
{
	unsigned int temp;

	if(voltage < 0){												//if value <0Volts 
		voltage = voltage * (-1);
		LcdChr(FONT_1X, '-');									//print "-" on screen
	}
	else
		LcdChr(FONT_1X, ' ');

	voltage = (voltage & 0xfffffffe);				
	temp = 0;		
	while(voltage >= 1000){									//calculate x0,00 (only x value) 
		voltage -=1000;
		temp +=1;
	}
	LcdChr(FONT_1X, temp + 48);							//print x0,00	(only x value) 	

	temp = 0;
	while(voltage >= 100){									//calculate 0x,00 (only x value) 
		voltage -=100;
		temp +=1;
	}
	LcdChr(FONT_1X, temp + 48);							//print 0x,00 (only x value) 

	LcdChr(FONT_1X, '.');										//print "."

	temp = 0;
	while(voltage >= 10){										//calculate 00,x0 (only x value) 
		voltage -=10;
		temp +=1;
	}
	LcdChr(FONT_1X, temp + 48);							//print 00,x0 (only x value) 
	LcdChr(FONT_1X, voltage + 48);					//print 00,0x (only x value) 
	LcdChr(FONT_1X,'v');										//print "v"
}

/*--------------------------------------------------------------------------------------------
  Name         :  ShowDutyCycle
  Description  :  display %dutycycle on lcd
  Time		   	 :  09/06/2009
  Notes        :    
---------------------------------------------------------------------------------------------*/	 
void ShowDutyCycle(void)
{
	unsigned int temp;

	dutycycle >>= (8+CD);											//if clock is lower then samples are 
	temp = 0;
	while(dutycycle >= 10){								   	//culculate tens, the rest is the monads
		dutycycle -=10;
		temp +=1;
	}
	LcdGotoXY(3,8);
	LcdChr(FONT_1X, temp + 48);								//display tens
	LcdGotoXY(4,8);	
	LcdChr(FONT_1X, dutycycle + 48);					//display monads 
	LcdChr(FONT_1X, '%');											//display "%" character
}

/*--------------------------------------------------------------------------------------------
  Name         :  SetFrequency
  Description  :  Set frequency and update lcd
  Time		   	 :  06/2009
  Notes        : 	Example: multiplier pre calculation for 75MHz external oscillator.
									fout = DPhase * ad9833clock/2^28 => DPhase = fout * 2^28/ad9833clock => DPhase = fout * 3,57913941...     
									We dont want to use floating point arithmetic so 
									multiplier = ((2^28 /ad9833clock)<<16) = 3,57913941... <<16 = 234562 integer number.
									DPhase = (fout * multiplier) >>16 =>  freqregister = (frequency * multiplier) >>16.  
---------------------------------------------------------------------------------------------*/
void SetFrequency(void)
{
	 unsigned int i;
	 unsigned long long freqregister = 0, frequency = 0;  		//be carefull with overflow
	 int m = 1;

	 for(i=0; i<7; i++){																			//calculate output frequency from digit[] array
	  	frequency += (digit[i] * m);
		m = m * 10;
	 }
	 freqregister = (frequency * multiplier);									//calculate AD9833 28bit frequency register
	 SetAD9833(freqregister >> 16 , FREQ0);										//and set frequency register

	  LcdGotoXY(1,3);																					//update lcd with current frequency
	  for(i=0; i<7; i++)
	  	  LcdChr(FONT_2X,digit[6-i]+32+16);
	  LcdUpdate();
}

/*--------------------------------------------------------------------------------------------
  Name         :  SetAD9833
  Description  :  Set frequency register in AD9834 chip. This function also works with AD9833 chip 
  Time		   	 :  06/2009
  Notes        : 	SPI interface used. We could use this function to change every register frequency,
									phase,contor inside AD9833,AD9834 chip.
---------------------------------------------------------------------------------------------*/
void SetAD9833(unsigned long  value , unsigned int mode)
{
	unsigned long int temp;	
	
	 temp = 0;	 	
	 switch (mode) {												   								//set frequency registers. Here mode is local variable
	  	case FREQ0 : case FREQ1 :
			temp = (((value & 0x3f00) | mode) >> 8);
			SPITX = temp;
			temp = (value & 0x000000ff);
			while((SPISTA & 0x01) != 0);
			SPITX = temp;
			temp = ((((value >> 14) & 0x3f00) | mode) >> 8);
			while((SPISTA & 0x01) != 0);
			SPITX = temp;
			temp = ((value >> 14) & 0x000000ff);
			while((SPISTA & 0x01) != 0);
			SPITX = temp;
			while((SPISTA & 0x01) != 0);
			break;
		case PHASE0 : case PHASE1 :									  						//set phase registers
			temp = (((value & 0x1f00) | mode) >> 8);
			SPITX = temp;
			temp = (value & 0x000000ff);
			while((SPISTA & 0x01) != 0);
			SPITX = temp;
			while((SPISTA & 0x01) != 0);		
			break;
		case CONTROL :											  										//set output waveform
			temp = (((value & 0x3f00) | mode )>> 8);
			SPITX = temp;
			temp = (value & 0x000000ff);
			while((SPISTA & 0x01) != 0);
			SPITX = temp;
			while((SPISTA & 0x01) != 0);		
			break;
		}
}

/*--------------------------------------------------------------------------------------------
  Name         :  Initialization
  Description  :  arm clock,lcd,logo screen,init screen,AD9833 spi,push buttons,ADC,1000Hz output
  Time		   	 :  06/2009
  Notes        : 
---------------------------------------------------------------------------------------------*/
void Initialization(void)
{
	unsigned int j,i,pwr,temp;

//set arm main clock
	POWKEY1 = 0x01;										   				//set arm clock
	POWCON = (0x00 | CD);												//if cd=0 then clock is 41.78MHz
	POWKEY2 = 0xf4;
	Delay(1000);
//init lcd
	InitLcd();
	LcdClear();
	LCDWrite130x130bmp();
	WriteLcdCommand(DISPON);
	Delay(2000000);
//++power on/off counter 
	FEEMOD = 0x08;
	temp = POWERON;														//read on/off counter
	FEEDAT = ++temp;								   				//increase on/off counter
	FEEADR = 0x8f000;
	FEECON = 0x03;														//erase page and write
	Delay(200000);									  				//flash delay time
	FEEMOD = 0x00;

//display firmware and powercycles on first screen
	LcdGotoXY(1,7);
	LcdStr(FONT_1X,firmware);
	LcdGotoXY(1,8);
	LcdStr(FONT_1X,powercycles);
	LcdGotoXY(7,8);
	pwr = POWERON;
	temp = 0;
	while(pwr >= 1000){												//calculate thousands
		pwr -=1000;
		temp +=1;
	}
	LcdChr(FONT_1X, temp + 48);								//display thousands
	temp = 0;
	while(pwr >= 100){
		pwr -=100;
		temp +=1;
	}
	LcdChr(FONT_1X, temp + 48);								//calculate hundreds
	temp = 0;
	while(pwr >= 10){
		pwr -=10;
		temp +=1;
	}
	LcdChr(FONT_1X, temp + 48);								//display hundreds
	LcdChr(FONT_1X, pwr + 48);								//display monads
	LcdUpdate();
	Delay(300000);
	
//fake boot effect ....
	for(i=0; i<4; i++){
		LcdChr(FONT_1X, '.');	
		Delay(1000000);
		LcdUpdate();
	}
	
//init screen
	LcdClear();
	LcdGotoXY(15,3);
	LcdStr(FONT_1X,initscreen);
	LcdUpdate();

//spi init for ad9834 chip
	temp = GP1CON;
	GP1CON = ((temp & 0x0000ffff) | 0x22220000); 			//pin1.4, pin1.5, pin1.6 pin1.7 SPI pins
	SPICON  = 0x10cb;																	//??? be carefull 
//GPIO init push buttons(input pins)
	temp = GP4CON; 
	GP4CON = (temp & 0xffffffcc); 								//pin4.0,pin4.1,pin4.2,pin4.3,pin4.4,pin4.5,pin4.6,,pin4.7 GPIO pins
	temp = GP4DAT;
	GP4DAT = (temp & 0x00ffffff);									//pin4.0,pin4.1,pin4.2,pin4.3,pin4.4,pin4.5,pin4.6,,pin4.7 input pins
//ADC init for measuremets
	REFCON = 0x01;																//internal band gap reference is used by ADC														
	ADCCP = 0x00;																	//ADC0 Positive channel	
	ADCCN = 0x01;																	//ADC1 Negative channel
	ADCCON = 0x604;																//fADC/1,8 clocks asquisition time,single-ended mode,continuous software conversion

	SetAD9833(RESET, CONTROL);										//reset AD9834
	Delay(10);
	SetAD9833(SINUSF0P0, CONTROL);								//set sinus 

	temp = GP1CON;																//P1.2 used to ADG701 chip
	GP1CON = (temp & 0xfffff0ff); 								//P1.2 GPIO pin
	temp = GP1DAT;
	GP1DAT = (temp | 0x04000000); 	 							//P1.2 output pin
	
	GP1CLR = 0x00040000;													//P1.2 clear for sinus output
	
//select wave type and frequency value on boot 																			
	for(i=0; i<7; i++){														//fill digit[] with 1000, fout = 1000Hz on boot 
		digit[i] = 0;
	}
	digit[3] = 1;
	mode = 0;																			//select sinus on boot
	pnt = 0;																			//cursor at frequency monads 

//sweep parameters initialization
	for(i=0; i<3; i++){
		for(j=0; j<7; j++)
			sweeparam[i][j]=0;
	}
	sweeparam[0][2]=1;														//start = 100Hz
	sweeparam[1][5]=1;														//stop = 10KHz
	sweeparam[2][2]=1;														//time = 100msec

	SetFrequency();																//set 1000Hz init output frequency
	SetMode();																		//set sinus wave

	ADCCON = 0x4a4;																//start measurements
	Delay(10);
}

/*--------------------------------------------------------------------------------------------
  Name         :  Delay
  Description  :  time delay in usec
  Time		   	 :  01/2008
  Notes        :  This fuction works on both modes(arm,thumb)
---------------------------------------------------------------------------------------------*/
void Delay(int usec)
{
	 int i;
	 int loops = (((usec<<MULTIPLIER) * 24 )-82);
     	 for(i=0; i<loops; i=i+23);
}

